# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
AWSTemplateFormatVersion: "2010-09-09"
Description: deployment of product {{ launch_details.product }} {{ launch_details.version }}

Resources:
  CloudFormationProvisionedProduct:
    Type: AWS::ServiceCatalog::CloudFormationProvisionedProduct
    Properties:
      ProductId: {{ launch_details.product_id }}
      ProvisioningArtifactId: {{ launch_details.provisioning_artifact_id }}
      ProvisionedProductName: {{ launch_details.launch_name }}
